@@ -1,34 +1,29 @@
 use std::char;
 use std::collections::{
-    BTreeMap, BTreeSet,
-    BinaryHeap,
-    HashMap, HashSet,
-    LinkedList,
-    VecDeque,
+    BTreeMap, BTreeSet, BinaryHeap, HashMap, HashSet, LinkedList, VecDeque,
 };
 use std::env;
 use std::ffi::OsString;
 use std::hash::{BuildHasher, Hash};
 use std::iter::{empty, once};
 use std::net::{
-    IpAddr, Ipv4Addr, Ipv6Addr,
-    SocketAddr, SocketAddrV4, SocketAddrV6,
+    IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr, SocketAddrV4, SocketAddrV6,
 };
 use std::num::Wrapping;
-use std::ops::{Range, RangeFrom, RangeTo, RangeFull};
+use std::ops::{Bound, Range, RangeFrom, RangeFull, RangeTo};
 use std::path::PathBuf;
 use std::sync::Arc;
-use std::time::{UNIX_EPOCH, Duration, SystemTime};
+use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
-use rand::{self, Rng, RngCore};
 use rand::seq::SliceRandom;
+use rand::{self, Rng, RngCore};
 
 /// `Gen` wraps a `rand::RngCore` with parameters to control the distribution of
 /// random values.
 ///
 /// A value with type satisfying the `Gen` trait can be constructed with the
 /// `gen` function in this crate.
-pub trait Gen : RngCore {
+pub trait Gen: RngCore {
     fn size(&self) -> usize;
 }
 
@@ -55,19 +50,27 @@ impl<R: RngCore> StdGen<R> {
 }
 
 impl<R: RngCore> RngCore for StdGen<R> {
-    fn next_u32(&mut self) -> u32 { self.rng.next_u32() }
+    fn next_u32(&mut self) -> u32 {
+        self.rng.next_u32()
+    }
 
     // some RNGs implement these more efficiently than the default, so
     // we might as well defer to them.
-    fn next_u64(&mut self) -> u64 { self.rng.next_u64() }
-    fn fill_bytes(&mut self, dest: &mut [u8]) { self.rng.fill_bytes(dest) }
+    fn next_u64(&mut self) -> u64 {
+        self.rng.next_u64()
+    }
+    fn fill_bytes(&mut self, dest: &mut [u8]) {
+        self.rng.fill_bytes(dest)
+    }
     fn try_fill_bytes(&mut self, dest: &mut [u8]) -> Result<(), rand::Error> {
         self.rng.try_fill_bytes(dest)
     }
 }
 
 impl<R: RngCore> Gen for StdGen<R> {
-    fn size(&self) -> usize { self.size }
+    fn size(&self) -> usize {
+        self.size
+    }
 }
 
 /// StdThreadGen is an RNG in thread-local memory.
@@ -88,28 +91,36 @@ impl StdThreadGen {
 }
 
 impl RngCore for StdThreadGen {
-    fn next_u32(&mut self) -> u32 { self.0.next_u32() }
+    fn next_u32(&mut self) -> u32 {
+        self.0.next_u32()
+    }
 
     // some RNGs implement these more efficiently than the default, so
     // we might as well defer to them.
-    fn next_u64(&mut self) -> u64 { self.0.next_u64() }
-    fn fill_bytes(&mut self, dest: &mut [u8]) { self.0.fill_bytes(dest) }
+    fn next_u64(&mut self) -> u64 {
+        self.0.next_u64()
+    }
+    fn fill_bytes(&mut self, dest: &mut [u8]) {
+        self.0.fill_bytes(dest)
+    }
     fn try_fill_bytes(&mut self, dest: &mut [u8]) -> Result<(), rand::Error> {
         self.0.try_fill_bytes(dest)
     }
 }
 
 impl Gen for StdThreadGen {
-    fn size(&self) -> usize { self.0.size }
+    fn size(&self) -> usize {
+        self.0.size
+    }
 }
 
 /// Creates a shrinker with zero elements.
-pub fn empty_shrinker<A: 'static>() -> Box<Iterator<Item=A>> {
+pub fn empty_shrinker<A: 'static>() -> Box<dyn Iterator<Item = A>> {
     Box::new(empty())
 }
 
 /// Creates a shrinker with a single element.
-pub fn single_shrinker<A: 'static>(value: A) -> Box<Iterator<Item=A>> {
+pub fn single_shrinker<A: 'static>(value: A) -> Box<dyn Iterator<Item = A>> {
     Box::new(once(value))
 }
 
@@ -125,26 +136,29 @@ pub fn single_shrinker<A: 'static>(value: A) -> Box<Iterator<Item=A>> {
 /// They must also be sendable and static since every test is run in its own
 /// thread using `thread::Builder::spawn`, which requires the `Send + 'static`
 /// bounds.
-pub trait Arbitrary : Clone + Send + 'static {
+pub trait Arbitrary: Clone + Send + 'static {
     fn arbitrary<G: Gen>(g: &mut G) -> Self;
 
-    fn shrink(&self) -> Box<Iterator<Item=Self>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Self>> {
         empty_shrinker()
     }
 }
 
 impl Arbitrary for () {
-    fn arbitrary<G: Gen>(_: &mut G) -> () { () }
+    fn arbitrary<G: Gen>(_: &mut G) -> () {
+        ()
+    }
 }
 
 impl Arbitrary for bool {
-    fn arbitrary<G: Gen>(g: &mut G) -> bool { g.gen() }
+    fn arbitrary<G: Gen>(g: &mut G) -> bool {
+        g.gen()
+    }
 
-    fn shrink(&self) -> Box<Iterator<Item=bool>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = bool>> {
         if *self {
             single_shrinker(false)
-        }
-        else {
+        } else {
             empty_shrinker()
         }
     }
@@ -159,7 +173,7 @@ impl<A: Arbitrary> Arbitrary for Option<A> {
         }
     }
 
-    fn shrink(&self)  -> Box<Iterator<Item=Option<A>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Option<A>>> {
         match *self {
             None => empty_shrinker(),
             Some(ref x) => {
@@ -179,7 +193,7 @@ impl<A: Arbitrary, B: Arbitrary> Arbitrary for Result<A, B> {
         }
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Result<A, B>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Result<A, B>>> {
         match *self {
             Ok(ref x) => {
                 let xs = x.shrink();
@@ -208,7 +222,7 @@ macro_rules! impl_arb_for_single_tuple {
                 )
             }
 
-            fn shrink(&self) -> Box<Iterator<Item=($($type_param,)*)>> {
+            fn shrink(&self) -> Box<dyn Iterator<Item=($($type_param,)*)>> {
                 let iter = ::std::iter::empty();
                 $(
                     let cloned = self.clone();
@@ -248,11 +262,14 @@ impl_arb_for_tuples! {
 
 impl<A: Arbitrary> Arbitrary for Vec<A> {
     fn arbitrary<G: Gen>(g: &mut G) -> Vec<A> {
-        let size = { let s = g.size(); g.gen_range(0, s) };
+        let size = {
+            let s = g.size();
+            g.gen_range(0, s)
+        };
         (0..size).map(|_| Arbitrary::arbitrary(g)).collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Vec<A>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Vec<A>>> {
         VecShrinker::new(self.clone())
     }
 }
@@ -266,15 +283,14 @@ struct VecShrinker<A> {
     offset: usize,
     /// The shrinker for the element at `offset` once shrinking of individual
     /// elements are attempted
-    element_shrinker: Box<Iterator<Item=A>>
+    element_shrinker: Box<dyn Iterator<Item = A>>,
 }
 
-impl <A: Arbitrary> VecShrinker<A> {
-
-    fn new(seed: Vec<A>) -> Box<Iterator<Item=Vec<A>>> {
+impl<A: Arbitrary> VecShrinker<A> {
+    fn new(seed: Vec<A>) -> Box<dyn Iterator<Item = Vec<A>>> {
         let es = match seed.get(0) {
             Some(e) => e.shrink(),
-            None => return empty_shrinker()
+            None => return empty_shrinker(),
         };
         let size = seed.len();
         Box::new(VecShrinker {
@@ -291,34 +307,35 @@ impl <A: Arbitrary> VecShrinker<A> {
         loop {
             match self.element_shrinker.next() {
                 Some(e) => return Some(e),
-                None => {
-                    match self.seed.get(self.offset) {
-                        Some(e) => {
-                            self.element_shrinker = e.shrink();
-                            self.offset += 1;
-                        }
-                        None => return None
+                None => match self.seed.get(self.offset) {
+                    Some(e) => {
+                        self.element_shrinker = e.shrink();
+                        self.offset += 1;
                     }
-                }
+                    None => return None,
+                },
             }
         }
     }
 }
 
-impl <A> Iterator for VecShrinker<A>
-    where A: Arbitrary {
+impl<A> Iterator for VecShrinker<A>
+where
+    A: Arbitrary,
+{
     type Item = Vec<A>;
     fn next(&mut self) -> Option<Vec<A>> {
         // Try with an empty vector first
         if self.size == self.seed.len() {
             self.size /= 2;
             self.offset = self.size;
-            return Some(vec![])
+            return Some(vec![]);
         }
         if self.size != 0 {
             // Generate a smaller vector by removing the elements between
             // (offset - size) and offset
-            let xs1 = self.seed[..(self.offset - self.size)].iter()
+            let xs1 = self.seed[..(self.offset - self.size)]
+                .iter()
                 .chain(&self.seed[self.offset..])
                 .cloned()
                 .collect();
@@ -330,8 +347,7 @@ impl <A> Iterator for VecShrinker<A>
                 self.offset = self.size;
             }
             Some(xs1)
-        }
-        else {
+        } else {
             // A smaller vector did not work so try to shrink each element of
             // the vector instead Reuse `offset` as the index determining which
             // element to shrink
@@ -339,14 +355,20 @@ impl <A> Iterator for VecShrinker<A>
             // The first element shrinker is already created so skip the first
             // offset (self.offset == 0 only on first entry to this part of the
             // iterator)
-            if self.offset == 0 { self.offset = 1 }
+            if self.offset == 0 {
+                self.offset = 1
+            }
 
             match self.next_element() {
-                Some(e) => Some(self.seed[..self.offset-1].iter().cloned()
-                    .chain(Some(e).into_iter())
-                    .chain(self.seed[self.offset..].iter().cloned())
-                    .collect()),
-                None => None
+                Some(e) => Some(
+                    self.seed[..self.offset - 1]
+                        .iter()
+                        .cloned()
+                        .chain(Some(e).into_iter())
+                        .chain(self.seed[self.offset..].iter().cloned())
+                        .collect(),
+                ),
+                None => None,
             }
         }
     }
@@ -358,27 +380,28 @@ impl<K: Arbitrary + Ord, V: Arbitrary> Arbitrary for BTreeMap<K, V> {
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=BTreeMap<K, V>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = BTreeMap<K, V>>> {
         let vec: Vec<(K, V)> = self.clone().into_iter().collect();
-        Box::new(vec.shrink()
-                    .map(|v| v.into_iter().collect::<BTreeMap<K, V>>()))
+        Box::new(
+            vec.shrink().map(|v| v.into_iter().collect::<BTreeMap<K, V>>()),
+        )
     }
 }
 
 impl<
-    K: Arbitrary + Eq + Hash,
-    V: Arbitrary,
-    S: BuildHasher + Default + Clone + Send + 'static,
-> Arbitrary for HashMap<K, V, S> {
+        K: Arbitrary + Eq + Hash,
+        V: Arbitrary,
+        S: BuildHasher + Default + Clone + Send + 'static,
+    > Arbitrary for HashMap<K, V, S>
+{
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         let vec: Vec<(K, V)> = Arbitrary::arbitrary(g);
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Self>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Self>> {
         let vec: Vec<(K, V)> = self.clone().into_iter().collect();
-        Box::new(vec.shrink()
-                    .map(|v| v.into_iter().collect::<Self>()))
+        Box::new(vec.shrink().map(|v| v.into_iter().collect::<Self>()))
     }
 }
 
@@ -388,7 +411,7 @@ impl<T: Arbitrary + Ord> Arbitrary for BTreeSet<T> {
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=BTreeSet<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = BTreeSet<T>>> {
         let vec: Vec<T> = self.clone().into_iter().collect();
         Box::new(vec.shrink().map(|v| v.into_iter().collect::<BTreeSet<T>>()))
     }
@@ -400,23 +423,25 @@ impl<T: Arbitrary + Ord> Arbitrary for BinaryHeap<T> {
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=BinaryHeap<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = BinaryHeap<T>>> {
         let vec: Vec<T> = self.clone().into_iter().collect();
-        Box::new(vec.shrink()
-                    .map(|v| v.into_iter().collect::<BinaryHeap<T>>()))
+        Box::new(
+            vec.shrink().map(|v| v.into_iter().collect::<BinaryHeap<T>>()),
+        )
     }
 }
 
 impl<
-    T: Arbitrary + Eq + Hash,
-    S: BuildHasher + Default + Clone + Send + 'static,
-> Arbitrary for HashSet<T, S> {
+        T: Arbitrary + Eq + Hash,
+        S: BuildHasher + Default + Clone + Send + 'static,
+    > Arbitrary for HashSet<T, S>
+{
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         let vec: Vec<T> = Arbitrary::arbitrary(g);
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Self>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Self>> {
         let vec: Vec<T> = self.clone().into_iter().collect();
         Box::new(vec.shrink().map(|v| v.into_iter().collect::<Self>()))
     }
@@ -428,10 +453,11 @@ impl<T: Arbitrary> Arbitrary for LinkedList<T> {
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=LinkedList<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = LinkedList<T>>> {
         let vec: Vec<T> = self.clone().into_iter().collect();
-        Box::new(vec.shrink()
-                    .map(|v| v.into_iter().collect::<LinkedList<T>>()))
+        Box::new(
+            vec.shrink().map(|v| v.into_iter().collect::<LinkedList<T>>()),
+        )
     }
 }
 
@@ -441,10 +467,9 @@ impl<T: Arbitrary> Arbitrary for VecDeque<T> {
         vec.into_iter().collect()
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=VecDeque<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = VecDeque<T>>> {
         let vec: Vec<T> = self.clone().into_iter().collect();
-        Box::new(vec.shrink()
-                    .map(|v| v.into_iter().collect::<VecDeque<T>>()))
+        Box::new(vec.shrink().map(|v| v.into_iter().collect::<VecDeque<T>>()))
     }
 }
 
@@ -467,8 +492,16 @@ impl Arbitrary for Ipv4Addr {
 
 impl Arbitrary for Ipv6Addr {
     fn arbitrary<G: Gen>(g: &mut G) -> Ipv6Addr {
-        Ipv6Addr::new(g.gen(), g.gen(), g.gen(), g.gen(),
-                      g.gen(), g.gen(), g.gen(), g.gen())
+        Ipv6Addr::new(
+            g.gen(),
+            g.gen(),
+            g.gen(),
+            g.gen(),
+            g.gen(),
+            g.gen(),
+            g.gen(),
+            g.gen(),
+        )
     }
 }
 
@@ -486,8 +519,7 @@ impl Arbitrary for SocketAddrV4 {
 
 impl Arbitrary for SocketAddrV6 {
     fn arbitrary<G: Gen>(g: &mut G) -> SocketAddrV6 {
-        SocketAddrV6::new(Arbitrary::arbitrary(g),
-            g.gen(), g.gen(), g.gen())
+        SocketAddrV6::new(Arbitrary::arbitrary(g), g.gen(), g.gen(), g.gen())
     }
 }
 
@@ -495,12 +527,11 @@ impl Arbitrary for PathBuf {
     fn arbitrary<G: Gen>(g: &mut G) -> PathBuf {
         // use some real directories as guesses, so we may end up with
         // actual working directories in case that is relevant.
-        let here = env::current_dir()
-            .unwrap_or(PathBuf::from("/test/directory"));
+        let here =
+            env::current_dir().unwrap_or(PathBuf::from("/test/directory"));
         let temp = env::temp_dir();
         #[allow(deprecated)]
-        let home = env::home_dir()
-            .unwrap_or(PathBuf::from("/home/user"));
+        let home = env::home_dir().unwrap_or(PathBuf::from("/home/user"));
         let choices = &[
             here,
             temp,
@@ -515,7 +546,7 @@ impl Arbitrary for PathBuf {
         p
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=PathBuf>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = PathBuf>> {
         let mut shrunk = vec![];
         let mut popped = self.clone();
         if popped.pop() {
@@ -546,7 +577,7 @@ impl Arbitrary for OsString {
         OsString::from(String::arbitrary(g))
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=OsString>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = OsString>> {
         let mystring: String = self.clone().into_string().unwrap();
         Box::new(mystring.shrink().map(|s| OsString::from(s)))
     }
@@ -554,7 +585,10 @@ impl Arbitrary for OsString {
 
 impl Arbitrary for String {
     fn arbitrary<G: Gen>(g: &mut G) -> String {
-        let size = { let s = g.size(); g.gen_range(0, s) };
+        let size = {
+            let s = g.size();
+            g.gen_range(0, s)
+        };
         let mut s = String::with_capacity(size);
         for _ in 0..size {
             s.push(char::arbitrary(g));
@@ -562,7 +596,7 @@ impl Arbitrary for String {
         s
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=String>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = String>> {
         // Shrink a string by shrinking a vector of its characters.
         let chars: Vec<char> = self.chars().collect();
         Box::new(chars.shrink().map(|x| x.into_iter().collect::<String>()))
@@ -573,48 +607,77 @@ impl Arbitrary for char {
     fn arbitrary<G: Gen>(g: &mut G) -> char {
         let mode = g.gen_range(0, 100);
         match mode {
-            0...49 => {
+            0..=49 => {
                 // ASCII + some control characters
-                g.gen_range(0,0xB0) as u8 as char
+                g.gen_range(0, 0xB0) as u8 as char
             }
-            50...59 => {
+            50..=59 => {
                 // Unicode BMP characters
                 loop {
                     if let Some(x) = char::from_u32(g.gen_range(0, 0x10000)) {
-                        return x
+                        return x;
                     }
                     // ignore surrogate pairs
                 }
             }
-            60...84 => {
+            60..=84 => {
                 // Characters often used in programming languages
                 [
-                    ' ', ' ', ' ',
-                    '\t',
-                    '\n',
-                    '~', '`', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')',
-                    '_', '-', '=', '+','[', ']', '{', '}', ':', ';', '\'', '"',
-                    '\\', '|',',','<','>','.','/','?',
-                    '0', '1','2','3','4','5','6','7','8','9',
-                ].choose(g).unwrap().to_owned()
+                    ' ', ' ', ' ', '\t', '\n', '~', '`', '!', '@', '#', '$',
+                    '%', '^', '&', '*', '(', ')', '_', '-', '=', '+', '[',
+                    ']', '{', '}', ':', ';', '\'', '"', '\\', '|', ',', '<',
+                    '>', '.', '/', '?', '0', '1', '2', '3', '4', '5', '6',
+                    '7', '8', '9',
+                ]
+                .choose(g)
+                .unwrap()
+                .to_owned()
             }
-            85...89 => {
+            85..=89 => {
                 // Tricky Unicode, part 1
                 [
                     '\u{0149}', // a deprecated character
                     '\u{fff0}', // some of "Other, format" category:
-                    '\u{fff1}','\u{fff2}','\u{fff3}','\u{fff4}','\u{fff5}',
-                    '\u{fff6}','\u{fff7}','\u{fff8}','\u{fff9}','\u{fffA}',
-                    '\u{fffB}','\u{fffC}','\u{fffD}','\u{fffE}','\u{fffF}',
-                    '\u{0600}','\u{0601}','\u{0602}','\u{0603}',
-                    '\u{0604}','\u{0605}','\u{061C}',
-                    '\u{06DD}','\u{070F}','\u{180E}',
-                    '\u{110BD}', '\u{1D173}',
+                    '\u{fff1}',
+                    '\u{fff2}',
+                    '\u{fff3}',
+                    '\u{fff4}',
+                    '\u{fff5}',
+                    '\u{fff6}',
+                    '\u{fff7}',
+                    '\u{fff8}',
+                    '\u{fff9}',
+                    '\u{fffA}',
+                    '\u{fffB}',
+                    '\u{fffC}',
+                    '\u{fffD}',
+                    '\u{fffE}',
+                    '\u{fffF}',
+                    '\u{0600}',
+                    '\u{0601}',
+                    '\u{0602}',
+                    '\u{0603}',
+                    '\u{0604}',
+                    '\u{0605}',
+                    '\u{061C}',
+                    '\u{06DD}',
+                    '\u{070F}',
+                    '\u{180E}',
+                    '\u{110BD}',
+                    '\u{1D173}',
                     '\u{e0001}', // tag
-                    '\u{e0020}',//  tag space
-                    '\u{e000}', '\u{e001}', '\u{ef8ff}', // private use
-                    '\u{f0000}', '\u{ffffd}','\u{ffffe}', '\u{fffff}',
-                    '\u{100000}','\u{10FFFD}','\u{10FFFE}','\u{10FFFF}',
+                    '\u{e0020}', //  tag space
+                    '\u{e000}',
+                    '\u{e001}',
+                    '\u{ef8ff}', // private use
+                    '\u{f0000}',
+                    '\u{ffffd}',
+                    '\u{ffffe}',
+                    '\u{fffff}',
+                    '\u{100000}',
+                    '\u{10FFFD}',
+                    '\u{10FFFE}',
+                    '\u{10FFFF}',
                     // "Other, surrogate" characters are so that very special
                     // that they are not even allowed in safe Rust,
                     //so omitted here
@@ -622,26 +685,28 @@ impl Arbitrary for char {
                     '\u{1680}',
                     // other space characters are already covered by two next
                     // branches
-                ].choose(g).unwrap().to_owned()
+                ]
+                .choose(g)
+                .unwrap()
+                .to_owned()
             }
-            90...94 => {
+            90..=94 => {
                 // Tricky unicode, part 2
                 char::from_u32(g.gen_range(0x2000, 0x2070)).unwrap()
             }
-            95...99 => {
+            95..=99 => {
                 // Completely arbitrary characters
                 g.gen()
             }
-            _ => unreachable!()
+            _ => unreachable!(),
         }
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=char>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = char>> {
         Box::new((*self as u32).shrink().filter_map(char::from_u32))
     }
 }
 
-
 macro_rules! unsigned_shrinker {
     ($ty:ty) => {
         mod shrinker {
@@ -651,16 +716,15 @@ macro_rules! unsigned_shrinker {
             }
 
             impl UnsignedShrinker {
-                pub fn new(x: $ty) -> Box<Iterator<Item=$ty>> {
+                pub fn new(x: $ty) -> Box<dyn Iterator<Item = $ty>> {
                     if x == 0 {
                         super::empty_shrinker()
                     } else {
-                        Box::new(vec![0].into_iter().chain(
-                            UnsignedShrinker {
-                                x: x,
-                                i: x / 2,
-                            }
-                        ))
+                        Box::new(
+                            vec![0]
+                                .into_iter()
+                                .chain(UnsignedShrinker { x: x, i: x / 2 }),
+                        )
                     }
                 }
             }
@@ -678,7 +742,7 @@ macro_rules! unsigned_shrinker {
                 }
             }
         }
-    }
+    };
 }
 
 macro_rules! unsigned_arbitrary {
@@ -691,7 +755,7 @@ macro_rules! unsigned_arbitrary {
                     use std::cmp::{min, max};
                     g.gen_range(0, max(1, min(s, $ty::max_value())))
                 }
-                fn shrink(&self) -> Box<Iterator<Item=$ty>> {
+                fn shrink(&self) -> Box<dyn Iterator<Item=$ty>> {
                     unsigned_shrinker!($ty);
                     shrinker::UnsignedShrinker::new(*self)
                 }
@@ -717,14 +781,11 @@ macro_rules! signed_shrinker {
             }
 
             impl SignedShrinker {
-                pub fn new(x: $ty) -> Box<Iterator<Item=$ty>> {
+                pub fn new(x: $ty) -> Box<dyn Iterator<Item = $ty>> {
                     if x == 0 {
                         super::empty_shrinker()
                     } else {
-                        let shrinker = SignedShrinker {
-                            x: x,
-                            i: x / 2,
-                        };
+                        let shrinker = SignedShrinker { x: x, i: x / 2 };
                         let mut items = vec![0];
                         if shrinker.i < 0 {
                             items.push(shrinker.x.abs());
@@ -747,7 +808,7 @@ macro_rules! signed_shrinker {
                 }
             }
         }
-    }
+    };
 }
 
 macro_rules! signed_arbitrary {
@@ -764,7 +825,7 @@ macro_rules! signed_arbitrary {
                     };
                     g.gen_range(lower, max(1, upper as $ty))
                 }
-                fn shrink(&self) -> Box<Iterator<Item=$ty>> {
+                fn shrink(&self) -> Box<dyn Iterator<Item=$ty>> {
                     signed_shrinker!($ty);
                     shrinker::SignedShrinker::new(*self)
                 }
@@ -783,9 +844,10 @@ signed_arbitrary! {
 
 impl Arbitrary for f32 {
     fn arbitrary<G: Gen>(g: &mut G) -> f32 {
-        let s = g.size(); g.gen_range(-(s as f32), s as f32)
+        let s = g.size();
+        g.gen_range(-(s as f32), s as f32)
     }
-    fn shrink(&self) -> Box<Iterator<Item=f32>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = f32>> {
         signed_shrinker!(i32);
         let it = shrinker::SignedShrinker::new(*self as i32);
         Box::new(it.map(|x| x as f32))
@@ -794,9 +856,10 @@ impl Arbitrary for f32 {
 
 impl Arbitrary for f64 {
     fn arbitrary<G: Gen>(g: &mut G) -> f64 {
-        let s = g.size(); g.gen_range(-(s as f64), s as f64)
+        let s = g.size();
+        g.gen_range(-(s as f64), s as f64)
     }
-    fn shrink(&self) -> Box<Iterator<Item=f64>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = f64>> {
         signed_shrinker!(i64);
         let it = shrinker::SignedShrinker::new(*self as i64);
         Box::new(it.map(|x| x as f64))
@@ -807,42 +870,65 @@ impl<T: Arbitrary> Arbitrary for Wrapping<T> {
     fn arbitrary<G: Gen>(g: &mut G) -> Wrapping<T> {
         Wrapping(T::arbitrary(g))
     }
-    fn shrink(&self) -> Box<Iterator<Item=Wrapping<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Wrapping<T>>> {
         Box::new(self.0.shrink().map(|inner| Wrapping(inner)))
     }
 }
 
+impl<T: Arbitrary> Arbitrary for Bound<T> {
+    fn arbitrary<G: Gen>(g: &mut G) -> Bound<T> {
+        match g.gen_range(0, 3) {
+            0 => Bound::Included(T::arbitrary(g)),
+            1 => Bound::Excluded(T::arbitrary(g)),
+            _ => Bound::Unbounded,
+        }
+    }
+    fn shrink(&self) -> Box<dyn Iterator<Item = Bound<T>>> {
+        match *self {
+            Bound::Included(ref x) => {
+                Box::new(x.shrink().map(Bound::Included))
+            }
+            Bound::Excluded(ref x) => {
+                Box::new(x.shrink().map(Bound::Excluded))
+            }
+            Bound::Unbounded => empty_shrinker(),
+        }
+    }
+}
+
 impl<T: Arbitrary + Clone + PartialOrd> Arbitrary for Range<T> {
     fn arbitrary<G: Gen>(g: &mut G) -> Range<T> {
-        Arbitrary::arbitrary(g) .. Arbitrary::arbitrary(g)
+        Arbitrary::arbitrary(g)..Arbitrary::arbitrary(g)
     }
-    fn shrink(&self) -> Box<Iterator<Item=Range<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Range<T>>> {
         Box::new(
-            (self.start.clone(), self.end.clone())
-            .shrink().map(|(s, e)| s .. e))
+            (self.start.clone(), self.end.clone()).shrink().map(|(s, e)| s..e),
+        )
     }
 }
 
 impl<T: Arbitrary + Clone + PartialOrd> Arbitrary for RangeFrom<T> {
     fn arbitrary<G: Gen>(g: &mut G) -> RangeFrom<T> {
-        Arbitrary::arbitrary(g) ..
+        Arbitrary::arbitrary(g)..
     }
-    fn shrink(&self) -> Box<Iterator<Item=RangeFrom<T>>> {
-        Box::new(self.start.clone().shrink().map(|start| start ..))
+    fn shrink(&self) -> Box<dyn Iterator<Item = RangeFrom<T>>> {
+        Box::new(self.start.clone().shrink().map(|start| start..))
     }
 }
 
 impl<T: Arbitrary + Clone + PartialOrd> Arbitrary for RangeTo<T> {
     fn arbitrary<G: Gen>(g: &mut G) -> RangeTo<T> {
-        .. Arbitrary::arbitrary(g)
+        ..Arbitrary::arbitrary(g)
     }
-    fn shrink(&self) -> Box<Iterator<Item=RangeTo<T>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = RangeTo<T>>> {
         Box::new(self.end.clone().shrink().map(|end| ..end))
     }
 }
 
 impl Arbitrary for RangeFull {
-    fn arbitrary<G: Gen>(_: &mut G) -> RangeFull { .. }
+    fn arbitrary<G: Gen>(_: &mut G) -> RangeFull {
+        ..
+    }
 }
 
 impl Arbitrary for Duration {
@@ -852,11 +938,12 @@ impl Arbitrary for Duration {
         Duration::new(seconds, nanoseconds)
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Self>> {
-        Box::new((self.as_secs(), self.subsec_nanos()).shrink()
-            .map(|(secs, nanos)| {
-                Duration::new(secs, nanos % 1_000_000)
-            }))
+    fn shrink(&self) -> Box<dyn Iterator<Item = Self>> {
+        Box::new(
+            (self.as_secs(), self.subsec_nanos())
+                .shrink()
+                .map(|(secs, nanos)| Duration::new(secs, nanos % 1_000_000)),
+        )
     }
 }
 
@@ -865,7 +952,7 @@ impl<A: Arbitrary> Arbitrary for Box<A> {
         Box::new(A::arbitrary(g))
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Box<A>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Box<A>>> {
         Box::new((**self).shrink().map(Box::new))
     }
 }
@@ -875,7 +962,7 @@ impl<A: Arbitrary + Sync> Arbitrary for Arc<A> {
         Arc::new(A::arbitrary(g))
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Arc<A>>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Arc<A>>> {
         Box::new((**self).shrink().map(Arc::new))
     }
 }
@@ -891,34 +978,30 @@ impl Arbitrary for SystemTime {
         }
     }
 
-    fn shrink(&self) -> Box<Iterator<Item=Self>> {
+    fn shrink(&self) -> Box<dyn Iterator<Item = Self>> {
         let duration = match self.duration_since(UNIX_EPOCH) {
             Ok(duration) => duration,
             Err(e) => e.duration(),
         };
-        Box::new(duration.shrink().flat_map(|d| {
-            vec![UNIX_EPOCH + d, UNIX_EPOCH - d]
-        }))
+        Box::new(
+            duration
+                .shrink()
+                .flat_map(|d| vec![UNIX_EPOCH + d, UNIX_EPOCH - d]),
+        )
     }
 }
 
 #[cfg(test)]
 mod test {
+    use super::Arbitrary;
     use rand;
     use std::collections::{
-        BTreeMap,
-        BTreeSet,
-        BinaryHeap,
-        HashMap,
-        HashSet,
-        LinkedList,
-        VecDeque,
+        BTreeMap, BTreeSet, BinaryHeap, HashMap, HashSet, LinkedList, VecDeque,
     };
     use std::fmt::Debug;
     use std::hash::Hash;
     use std::num::Wrapping;
     use std::path::PathBuf;
-    use super::Arbitrary;
 
     #[test]
     fn arby_unit() {
@@ -927,12 +1010,18 @@ mod test {
 
     #[test]
     fn arby_int() {
-        rep(&mut || { let n: isize = arby(); assert!(n >= -5 && n <= 5); } );
+        rep(&mut || {
+            let n: isize = arby();
+            assert!(n >= -5 && n <= 5);
+        });
     }
 
     #[test]
     fn arby_uint() {
-        rep(&mut || { let n: usize = arby(); assert!(n <= 5); } );
+        rep(&mut || {
+            let n: usize = arby();
+            assert!(n <= 5);
+        });
     }
 
     fn arby<A: super::Arbitrary>() -> A {
@@ -943,7 +1032,10 @@ mod test {
         super::StdGen::new(rand::thread_rng(), 5)
     }
 
-    fn rep<F>(f: &mut F) where F : FnMut() -> () {
+    fn rep<F>(f: &mut F)
+    where
+        F: FnMut() -> (),
+    {
         for _ in 0..100 {
             f()
         }
@@ -988,16 +1080,20 @@ mod test {
     fn triples() {
         eq((false, false, false), vec![]);
         eq((true, false, false), vec![(false, false, false)]);
-        eq((true, true, false),
-           vec![(false, true, false), (true, false, false)]);
+        eq(
+            (true, true, false),
+            vec![(false, true, false), (true, false, false)],
+        );
     }
 
     #[test]
     fn quads() {
         eq((false, false, false, false), vec![]);
         eq((true, false, false, false), vec![(false, false, false, false)]);
-        eq((true, true, false, false),
-            vec![(false, true, false, false), (true, false, false, false)]);
+        eq(
+            (true, true, false, false),
+            vec![(false, true, false, false), (true, false, false, false)],
+        );
     }
 
     #[test]
@@ -1081,7 +1177,7 @@ mod test {
 
     macro_rules! define_float_eq {
         ($ty:ty) => {
-            fn eq(s:$ty, v: Vec<$ty> ) {
+            fn eq(s: $ty, v: Vec<$ty>) {
                 let shrunk: Vec<$ty> = s.shrink().collect();
                 for n in v {
                     let found = shrunk.iter().any(|&i| i == n);
@@ -1089,11 +1185,12 @@ mod test {
                         panic!(format!(
                             "Element {:?} was not found \
                              in shrink results {:?}",
-                            n, shrunk));
+                            n, shrunk
+                        ));
                     }
                 }
             }
-        }
+        };
     }
 
     #[test]
@@ -1123,20 +1220,43 @@ mod test {
     #[test]
     fn wrapping_ints32() {
         eq(Wrapping(5i32), vec![Wrapping(0), Wrapping(3), Wrapping(4)]);
-        eq(Wrapping(-5i32), vec![Wrapping(5), Wrapping(0), Wrapping(-3), Wrapping(-4)]);
+        eq(
+            Wrapping(-5i32),
+            vec![Wrapping(5), Wrapping(0), Wrapping(-3), Wrapping(-4)],
+        );
         eq(Wrapping(0i32), vec![]);
     }
 
     #[test]
     fn vecs() {
-        eq({let it: Vec<isize> = vec![]; it}, vec![]);
-        eq({let it: Vec<Vec<isize>> = vec![vec![]]; it}, vec![vec![]]);
+        eq(
+            {
+                let it: Vec<isize> = vec![];
+                it
+            },
+            vec![],
+        );
+        eq(
+            {
+                let it: Vec<Vec<isize>> = vec![vec![]];
+                it
+            },
+            vec![vec![]],
+        );
         eq(vec![1isize], vec![vec![], vec![0]]);
         eq(vec![11isize], vec![vec![], vec![0], vec![6], vec![9], vec![10]]);
         eq(
             vec![3isize, 5],
-            vec![vec![], vec![5], vec![3], vec![0,5], vec![2,5],
-                 vec![3,0], vec![3,3], vec![3,4]]
+            vec![
+                vec![],
+                vec![5],
+                vec![3],
+                vec![0, 5],
+                vec![2, 5],
+                vec![3, 0],
+                vec![3, 3],
+                vec![3, 4],
+            ],
         );
     }
 
@@ -1152,14 +1272,22 @@ mod test {
 
                     let shrinks = vec![
                         $ctor,
-                        {let mut m = $ctor; m.insert(0, 1); m},
-                        {let mut m = $ctor; m.insert(1, 0); m},
+                        {
+                            let mut m = $ctor;
+                            m.insert(0, 1);
+                            m
+                        },
+                        {
+                            let mut m = $ctor;
+                            m.insert(1, 0);
+                            m
+                        },
                     ];
 
                     ordered_eq(map, shrinks);
                 }
             }
-        }
+        };
     }
 
     map_tests!(btreemap, BTreeMap::<usize, isize>::new());
@@ -1177,14 +1305,22 @@ mod test {
 
                     let shrinks = vec![
                         $ctor,
-                        {let mut m = $ctor; m.$push(0); m},
-                        {let mut m = $ctor; m.$push(1); m},
+                        {
+                            let mut m = $ctor;
+                            m.$push(0);
+                            m
+                        },
+                        {
+                            let mut m = $ctor;
+                            m.$push(1);
+                            m
+                        },
                     ];
 
                     ordered_eq(list, shrinks);
                 }
             }
-        }
+        };
     }
 
     list_tests!(btreesets, BTreeSet::<usize>::new(), insert);
@@ -1196,17 +1332,14 @@ mod test {
     fn binaryheaps() {
         ordered_eq(
             BinaryHeap::<usize>::new().into_iter().collect::<Vec<_>>(),
-            vec![]);
+            vec![],
+        );
 
         {
             let mut heap = BinaryHeap::<usize>::new();
             heap.push(2usize);
 
-            let shrinks = vec![
-                vec![],
-                vec![0],
-                vec![1],
-            ];
+            let shrinks = vec![vec![], vec![0], vec![1]];
 
             ordered_eq(heap.into_iter().collect::<Vec<_>>(), shrinks);
         }
@@ -1234,6 +1367,16 @@ mod test {
         assert_eq!(left, right);
     }
 
+    #[test]
+    fn bounds() {
+        use std::ops::Bound::*;
+        for i in -5..=5 {
+            ordered_eq(Included(i), i.shrink().map(Included).collect());
+            ordered_eq(Excluded(i), i.shrink().map(Excluded).collect());
+        }
+        eq(Unbounded::<i32>, vec![]);
+    }
+
     #[test]
     fn ranges() {
         ordered_eq(0..0, vec![]);
@@ -1247,9 +1390,12 @@ mod test {
 
     #[test]
     fn pathbuf() {
-        ordered_eq(PathBuf::from("/home/foo//.././bar"), vec![
-            PathBuf::from("/home/foo//.."),
-            PathBuf::from("/home/foo/../bar"),
-        ]);
+        ordered_eq(
+            PathBuf::from("/home/foo//.././bar"),
+            vec![
+                PathBuf::from("/home/foo//.."),
+                PathBuf::from("/home/foo/../bar"),
+            ],
+        );
     }
 }
